/*
  Project: NEO-6M GY-GPS6MV2 GPS module, 1.8" SPI TFT LCD 128x160 module
  Function: This sketch listen to the GPS serial port,
  and when data received from the module, it's displays it on 1.8" SPI TFT LCD 128x160 module.

  Wiring GPS module:
  GPS module -> Arduino Uno
  GND           GND
  RX pin        Digital pin 3
  TX pin        Digital pin 4
  VCC pin       5V

  Wiring 1.8" SPI TFT LCD 128x160 module:
  1.8" SPI TFT LCD 128x160 module -> Arduino Uno
  VCC           5V
  GND           GND
  CS            Pin 10
  RESET         Pin 8
  A0            Pin 9
  SDA           Pin 11
  SCK           Pin 13
  LED           3.3V
*/
//**********************************************************************************************
#include <TFT_ILI9163.h> // include library for ILI9163 based TFT displays
#include <SPI.h>         // include SPI library, allows you to communicate with SPI devices
#include <TimeLib.h>     // include Time library
#include <SoftwareSerial.h>//include library code to allow serial communication on other digital pins of the Arduino board
#include <TinyGPS++.h>     //include the library code for GPS module
//**********************************************************************************************
#define time_offset   10800  // define a clock offset of 10800 seconds (3 hours) ==> UTC + 3
//Constants defined here:
static const int RXPin = 4, TXPin = 3; // define GPS module RX and TX pins
static const uint32_t GPSBaud = 9600;  // define GPS baud rate
static const double WAYPOINT_LAT = 51.508131, WAYPOINT_LON = -0.128002; // change the waypoint coordinates
TinyGPSPlus gps;// define the TinyGPS++ object
TFT_ILI9163 tft = TFT_ILI9163(); // define the TFT_ILI9163 object
SoftwareSerial ss(RXPin, TXPin);// the serial connection to the GPS device
//Variables defined here:
char Time[]  = "00:00:00";
char Date[]  = "00-00-2000";
byte last_second, Second, Minute, Hour, Day, Month;
int Year;

void setup(void) {
  tft.init();   // initialize a ILI9163S chip
  tft.setTextWrap(false);
  tft.fillScreen(ILI9163_BLACK);
  tft.setTextSize(1); // defines the text size
  tft.setTextColor(ILI9163_WHITE); // set font color - white
  tft.setCursor(0, 0);
  tft.println("Arduino GPS-Timing"); // prints text on TFT LCD
  tft.setCursor(0, 10);
  tft.println("Check for more"); // prints text on TFT LCD
  tft.setCursor(0, 20);
  tft.println("DIY rojects on"); // prints text on TFT LCD
  tft.setCursor(0, 30);
  tft.println("http://acoptex.com"); // prints text on TFT LCD
  ss.begin(GPSBaud);
  delay(4000); // set delay for 4 seconds
  tft.fillScreen(ILI9163_BLACK);
}
void print_wday(byte wday)
{
  tft.setTextColor(ILI9163_WHITE);
  tft.setCursor(60, 30);
  switch (wday)
  {
    case 1:  tft.println("SUNDAY");   break;
    case 2:  tft.println("MONDAY");   break;
    case 3:  tft.println("TUESDAY");   break;
    case 4:  tft.println("WEDNESDAY");   break;
    case 5:  tft.println("THURSDAY");   break;
    case 6:  tft.println("FRIDAY");   break;
    default: tft.println("SATURDAY");
  }
}
void displayInfo()
{
  // get time from GPS module
  if (gps.time.isValid())
  {
    Minute = gps.time.minute();
    Second = gps.time.second();
    Hour   = gps.time.hour();
  }
  // get date drom GPS module
  if (gps.date.isValid())
  {
    Day   = gps.date.day();
    Month = gps.date.month();
    Year  = gps.date.year();
  }

  if (last_second != gps.time.second()) // if time has changed
  {
    last_second = gps.time.second();

    // set current UTC time
    setTime(Hour, Minute, Second, Day, Month, Year);
    // add the offset to get local time
    adjustTime(time_offset);

    // update time array
    Time[0]  = hour()   / 10 + '0';
    Time[1]  = hour()   % 10 + '0';
    Time[3]  = minute() / 10 + '0';
    Time[4] = minute() % 10 + '0';
    Time[6] = second() / 10 + '0';
    Time[7] = second() % 10 + '0';

    // update date array
    Date[0]  =  day()   / 10 + '0';
    Date[1]  =  day()   % 10 + '0';
    Date[3]  =  month() / 10 + '0';
    Date[4] =  month() % 10 + '0';
    Date[8] = (year()  / 10) % 10 + '0';
    Date[9] =  year()  % 10 + '0';
    // print time & date
    tft.setTextColor(ILI9163_GREEN);
    tft.setCursor(60, 10);
    tft.println(Date);
    tft.setCursor(60, 20);
    tft.println(Time);
    print_wday(weekday());   // print day of the week
  }
  if (gps.location.isValid())
  {
    String Latitude = String(gps.location.lat(), 6);
    String Longitude = String(gps.location.lng(), 6);
    tft.setTextColor(ILI9163_GREEN);
    tft.setCursor(60, 40);
    tft.println(Latitude);
    tft.setCursor(60, 50);
    tft.println(Longitude);
  }
  if (gps.altitude.isValid())
  {
    String alt = String(gps.altitude.meters(), 2);
    tft.setTextColor(ILI9163_WHITE);
    tft.setCursor(60, 60);
    tft.println(alt);
  }
  if (gps.satellites.isValid())
  {
    String sat = String(gps.satellites.value());
    tft.setTextColor(ILI9163_WHITE);
    tft.setCursor(70, 70);
    tft.println(sat);
  }
  if (gps.location.isValid())
  {
    tft.setTextColor(ILI9163_WHITE);
    tft.setCursor(70, 80);
    tft.println("OK");
  }
  if (gps.speed.isValid())
  {
    String speed1 = String(gps.speed.kmph(), 2);
    tft.setTextColor(ILI9163_BLUE);
    tft.setCursor(70, 90);
    tft.println(speed1);
  }
  if (gps.course.isValid())
  {
    String course1 = String(gps.course.deg(), 2);
    tft.setTextColor(ILI9163_BLUE);
    tft.setCursor(70, 100);
    tft.println(course1);
  }
  if (gps.location.isValid())
  {
    unsigned long distanceKmToWaypoint =
      (unsigned long)TinyGPSPlus::distanceBetween(
        gps.location.lat(),
        gps.location.lng(),
        WAYPOINT_LAT,
        WAYPOINT_LON) / 1000;
    double courseToWaypoint =
      TinyGPSPlus::courseTo(
        gps.location.lat(),
        gps.location.lng(),
        WAYPOINT_LAT,
        WAYPOINT_LON);
    String course1 = String(gps.course.deg(), 2);
    const char *cardinalToWaypoint = TinyGPSPlus::cardinal(courseToWaypoint);
    tft.setTextColor(ILI9163_GREEN);
    tft.setCursor(70, 110);
    tft.println(distanceKmToWaypoint);
    tft.setCursor(70, 120);
    tft.println(courseToWaypoint);
    tft.setCursor(70, 130);
    tft.println(*cardinalToWaypoint);
  }
}

void loop()
{
  //displays information every time a new sentence is correctly encoded.
  while (ss.available() > 0)
    if (gps.encode(ss.read()))
      tft.fillScreen(ILI9163_BLACK);
  tft.setTextColor(ILI9163_WHITE);
  tft.setCursor(0, 0);
  tft.println("**ACOPTEX GPS Clock**");
  tft.setCursor(0, 10);
  tft.setTextColor(ILI9163_GREEN);
  tft.println("DATE: ");
  tft.setCursor(0, 20);
  tft.println("TIME: ");
  tft.setTextColor(ILI9163_WHITE);
  tft.setCursor(0, 30);
  tft.println("DAY OF WK:");
  tft.setTextColor(ILI9163_GREEN);
  tft.setCursor(0, 40);
  tft.println("LATITUDE:");
  tft.setCursor(0, 50);
  tft.println("LONGITUDE:");
  tft.setTextColor(ILI9163_WHITE);
  tft.setCursor(0, 60);
  tft.println("ALTITUDE:");
  tft.setCursor(0, 70);
  tft.println("SATELLITES:");
  tft.setCursor(0, 80);
  tft.println("FIX AGE:");
  tft.setTextColor(ILI9163_BLUE);
  tft.setCursor(0, 90);
  tft.println("SPEED km/h:");
  tft.setCursor(0, 100);
  tft.println("COURSE deg:");
  tft.setTextColor(ILI9163_GREEN);
  tft.setCursor(0, 110);
  tft.println("DIST WPT km:");
  tft.setCursor(0, 120);
  tft.println("HDG WPT deg:");
  tft.setCursor(0, 130);
  tft.println("CARD WPT:");
  displayInfo();
  if (millis() > 5000 && gps.charsProcessed() < 10)
  {
    tft.fillScreen(ILI9163_BLACK);
    tft.setTextSize(1);
    tft.setTextColor(ILI9163_WHITE);
    tft.setCursor(0, 0);
    tft.println("No GPS detected:");
    tft.setCursor(0, 10);
    tft.println(" check wiring.");
    tft.setCursor(0, 20);
    tft.println("More projects are");
    tft.setCursor(0, 30);
    tft.println(" on Acoptex.com");
    while (true);
  }
  delay(100);
}
